==> en-select-1.0.5.tgz: package/addon/components/en-select.js <==
import Component from '@ember/component'
import { A } from '@ember/array'
import { run } from '@ember/runloop'
import { isEmpty } from '@ember/utils'
import {
  computed,
  set,
  get,
  getWithDefault,
  getProperties,
} from '@ember/object'

import ScrollableList from 'en-select/mixins/scrollable-list'
import KeyMap from 'en-select/utils/keymap'
import basicSearch from 'en-select/utils/basic-search'
import { fallback } from 'en-select/utils/fallback'

export default Component.extend(ScrollableList, {
  classNames: ['en-select'],
  classNameBindings: [
    'isOpen:is-open',
    'isFixed:is-fixed',
    'isDisabled:is-disabled',
    'hasSelections:has-selections',
  ],

  attributeBindings: ['tabindex', 'role'],
  documentEvents: ['click', 'resize', 'scroll'],

  tabindex: 0,
  role: 'listbox',

  multiple: false,

  canRemoveSelection: true,

  optionLabelPath: 'label',
  optionValuePath: 'id',

  isOpen: false,
  isDisabled: false,
  isClosing: false,
  isFixed: false,

  positionFromTop: false,
  positionFromRight: false,

  options: fallback(A([])),
  selections: fallback(A([])),
  focused: null,

  search: null,
  canSearch: true,

  label: 'Search...',

  init() {
    this._super(...arguments)

    let { options, selections } = getProperties(this, 'options', 'selections')

    if (!options || isEmpty(options)) {
      set(this, 'options', A([]))
    }

    if (!selections || isEmpty(selections)) {
      set(this, 'selections', A([]))
    }

    this._syncSelectionsWithValue()
  },

  didReceiveAttrs() {
    this._syncSelectionsWithValue()
  },

  keyUp(event) {
    this._super(...arguments)

    const { which, keyCode } = event
    const key = which || keyCode

    const isOpen = get(this, 'isOpen')

    switch(key) {
      case KeyMap.enter:
        if (!isOpen) {
          this.openSelect()
        } else {
          this.selectOption(get(this, 'focused'), event)
        }

        break;

      case KeyMap.esc:
        if (isOpen) this.closeSelect()
        break;

      default:
        break;
    }
  },

  hasSelections: computed.gt('selections.length', 0),

  selection: computed('selections.[]', {
    get() {
      return get(this, 'selections.firstObject')
    },
  }),

  optionsThatMatchSearch: computed('search', 'options.[]', {
    get() {
      const { options, search } = getProperties(this, 'options', 'search')
      const labelPath = getWithDefault(this, 'optionLabelPath', 'label')

      if (isEmpty(search)) {
        return options
      } else {
        return basicSearch(search, options, { labelPath })
      }
    }
  }),

  selectableOptions: computed.setDiff('optionsThatMatchSearch', 'selections'),

  searchFor(term) {
    set(this, 'search', term)

    run(() => {
      this.focusOnFirstOption()
      this.scrollIfFocusNotVisible()
    })
  },

  /**
   * openSelect
   *
   * Opens the dropdown and focuses on the search box
   */
  openSelect() {
    if (get(this, 'isDisabled')) return

    run(() => {
      set(this, 'isOpen', true)
      this._focusOnSearch()
      this.focusOnFirstOption()
    })
  },

  /**
   * closeSelect
   *
   * Closes the dropdown
   */

  closeSelect() {
    run(() => {
      if (get(this, 'isDestroyed')) return
      set(this, 'isOpen', false)
    })
  },

  /**
   * focusOption
   *
   * Sets the focus to a given option
   */

  focusOption(option) {
    set(this, 'focused', option)

    if (this.attrs['on-focus']) {
      this.attrs['on-focus'](option)
    }
  },

  /**
   * selectOption
   *
   * Selects a given option. There are a couple of scenarios
   * with different behaviors:
   *
   * (a) when it's a multi-select, we select the option, fire
   * an action, and set the focus on search (i.e. don't close
   * the dropdown)
   *
   * (b) when it's a single-select, select the option and close
   * the dropdown.
   *
   * Finally, set the search and focus to null.
   */

  selectOption(option, event) {
    event.stopPropagation()

    let multiple = get(this, 'multiple')

    this._select(option)
    this.attrs['on-select'](option)

    if (multiple) {
      this._focusOnSearch()

    } else {
      this.closeSelect()
      this.element.focus()
    }

    set(this, 'search', null)
    set(this, 'focused', null)
  },

  /**
   * removeSelection
   *
   * Removes a given option and fires an action.
   */

  removeSelection(option = null) {
    const multiple = get(this, 'multiple')

    if (multiple) {
      get(this, 'selections').removeObject(option)
    } else {
      set(this, 'selections', A([]))
    }

    if (this.attrs['on-remove']) {
      this.attrs['on-remove'](option)
    }
  },

  /**
   * @private
   * _syncSelectionsWithValues()
   *
   * Takes the value supplied to the component and sets the selections from it.
   */

  _syncSelectionsWithValue() {
    let value = get(this, 'value')
    let multiple = get(this, 'multiple')

    if (!value) return

    if (multiple) {
      value.forEach(value => this._select(value))
    } else {
      this._select(value)
    }
  },

  _select(option) {
    if (get(this, "isDestroyed")) return

    let { options, multiple, optionValuePath } = getProperties(
      this, "options", "multiple", "optionValuePath"
    )

    if (isEmpty(options) || isEmpty(option)) return

    let id

    /*
     * Figure out the ID of the object. If the value
     * we get is a string, we just match the IDs to
     * be sure.
     *
     * In case it's an object, we find the ID using the
     * `optionValuePath` and use that to find the selected
     * option.
    */

    if (typeof option === "object") {
      id = get(option, optionValuePath)
    } else {
      id = option
    }

    let optionsThatMatch = options.filter(option => {
      return get(option, optionValuePath) === id
    })

    if (multiple) {
      get(this, "selections").addObjects(optionsThatMatch)
    } else {
      set(this, "selections", A(optionsThatMatch))
    }
  },

  _focusOnSearch() {
    run.scheduleOnce('afterRender', this, () => {
      const searchBox = this.element.querySelector('.en-select-search')
      if (searchBox) searchBox.focus()
    })
  }
})

==> en-select-1.1.0.tgz: package/addon/components/en-select.js <==
import Component from '@ember/component'
import { A } from '@ember/array'
import { run } from '@ember/runloop'
import { isEmpty } from '@ember/utils'
import {
  computed,
  set,
  get,
  getProperties,
} from '@ember/object'

import Search from 'en-select/utils/search'
import ScrollableList from 'en-select/mixins/scrollable-list'
import DocumentEvent from 'en-select/mixins/document-event'

import KeyMap from 'en-select/utils/keymap'
import { fallback } from 'en-select/utils/fallback'

export default Component.extend(ScrollableList, DocumentEvent, {
  classNames: ['en-select'],
  classNameBindings: [
    'isOpen:is-open',
    'isFixed:is-fixed',
    'isDisabled:is-disabled',
    'hasSelections:has-selections',
  ],

  attributeBindings: ['tabindex', 'role'],
  documentEvents: ['keydown'],

  tabindex: 0,
  role: 'listbox',

  multiple: false,

  canRemoveSelection: true,

  optionLabelPath: 'label',
  optionValuePath: 'id',

  isOpen: false,
  isDisabled: false,
  isClosing: false,
  isFixed: false,

  positionFromTop: false,
  positionFromRight: false,

  options: fallback(A([])),
  selections: fallback(A([])),
  focused: null,

  search: null,
  canSearch: true,

  label: 'Search...',

  init() {
    this._super(...arguments)

    let { options, selections } = getProperties(this, 'options', 'selections')

    set(this, 'searchService', Search.create({}))

    if (!options || isEmpty(options)) {
      set(this, 'options', A([]))
    }

    if (!selections || isEmpty(selections)) {
      set(this, 'selections', A([]))
    }

    this._syncSelectionsWithValue()
  },

  didReceiveAttrs() {
    this._syncSelectionsWithValue()
    this._buildSearchIndex()
  },

  didUpdateAttrs() {
    this._buildSearchIndex()
  },

  onDocumentKeydown(event) {
    const isOpen = get(this, 'isOpen')

    const { which, keyCode } = event
    const key = which || keyCode

    if (isOpen && key === KeyMap.enter) {
      event.preventDefault()
    }
  },

  keyUp(event) {
    this._super(...arguments)

    const { which, keyCode } = event
    const key = which || keyCode

    const isOpen = get(this, 'isOpen')

    switch(key) {
      case KeyMap.enter:
        if (!isOpen) {
          this.openSelect()
        } else {
          this.selectOption(get(this, 'focused'), event)
        }

        break;

      case KeyMap.esc:
        if (isOpen) this.closeSelect()
        break;

      default:
        break;
    }
  },

  hasSelections: computed.gt('selections.length', 0),

  selection: computed('selections.[]', {
    get() {
      return get(this, 'selections.firstObject')
    },
  }),

  optionsThatMatchSearch: computed('search', 'options.[]', {
    get() {
      const searchService = get(this, 'searchService')
      const { options, search: term } = getProperties(this, 'options', 'search')

      if (isEmpty(term)) {
        return options
      } else {
        return searchService.searchFor(term)
      }
    }
  }),

  selectableOptions: computed.setDiff('optionsThatMatchSearch', 'selections'),

  searchFor(term) {
    set(this, 'search', term)

    run(() => {
      this.focusOnFirstOption()
      this.scrollIfFocusNotVisible()
    })
  },

  /**
   * openSelect
   *
   * Opens the dropdown and focuses on the search box
   */
  openSelect() {
    if (get(this, 'isDisabled')) return

    run(() => {
      set(this, 'isOpen', true)
      this._focusOnSearch()
      this.focusOnFirstOption()
    })
  },

  /**
   * closeSelect
   *
   * Closes the dropdown
   */

  closeSelect() {
    run(() => {
      if (get(this, 'isDestroyed')) return
      set(this, 'isOpen', false)
    })
  },

  /**
   * focusOption
   *
   * Sets the focus to a given option
   */

  focusOption(option) {
    set(this, 'focused', option)

    if (this.attrs['on-focus']) {
      this.attrs['on-focus'](option)
    }
  },

  /**
   * selectOption
   *
   * Selects a given option. There are a couple of scenarios
   * with different behaviors:
   *
   * (a) when it's a multi-select, we select the option, fire
   * an action, and set the focus on search (i.e. don't close
   * the dropdown)
   *
   * (b) when it's a single-select, select the option and close
   * the dropdown.
   *
   * Finally, set the search and focus to null.
   */

  selectOption(option, event) {
    event.stopPropagation()

    if (!option) {
      return
    }

    let multiple = get(this, 'multiple')

    this._select(option)
    this.attrs['on-select'](option)

    if (multiple) {
      this._focusOnSearch()

    } else {
      this.closeSelect()
      this.element.focus()
    }

    set(this, 'search', null)
    set(this, 'focused', null)
  },

  /**
   * removeSelection
   *
   * Removes a given option and fires an action.
   */

  removeSelection(option = null) {
    const multiple = get(this, 'multiple')

    if (multiple) {
      get(this, 'selections').removeObject(option)
    } else {
      set(this, 'selections', A([]))
    }

    if (this.attrs['on-remove']) {
      this.attrs['on-remove'](option)
    }
  },

  /**
   * @private
   * _syncSelectionsWithValues()
   *
   * Takes the value supplied to the component and sets the selections from it.
   */

  _syncSelectionsWithValue() {
    let value = get(this, 'value')
    let multiple = get(this, 'multiple')

    if (!value) return

    if (multiple) {
      value.forEach(value => this._select(value))
    } else {
      this._select(value)
    }
  },

  _select(option) {
    if (get(this, "isDestroyed")) return

    let { options, multiple, optionValuePath } = getProperties(
      this, "options", "multiple", "optionValuePath"
    )

    if (isEmpty(options) || isEmpty(option)) return

    let id

    /*
     * Figure out the ID of the object. If the value
     * we get is a string, we just match the IDs to
     * be sure.
     *
     * In case it's an object, we find the ID using the
     * `optionValuePath` and use that to find the selected
     * option.
    */

    if (typeof option === "object") {
      id = get(option, optionValuePath)
    } else {
      id = option
    }

    let optionsThatMatch = options.filter(option => {
      return get(option, optionValuePath) === id
    })

    if (multiple) {
      get(this, "selections").addObjects(optionsThatMatch)
    } else {
      set(this, "selections", A(optionsThatMatch))
    }
  },

  _focusOnSearch() {
    run.scheduleOnce('afterRender', this, () => {
      const searchBox = this.element.querySelector('.en-select-search')
      if (searchBox) searchBox.focus()
    })
  },

  _buildSearchIndex() {
    const options = get(this, 'options')
    const labelPath = get(this, 'optionLabelPath')
    const searchService = get(this, 'searchService')

    return searchService.buildIndexFor(options, labelPath)
  }
})
